
%====================================================================%
%
%7606C,双通道采集分离显示
%基于7015，搭载两个7606C模块，每个模块同时采集相同通道，后续可改不同通道
%
%====================================================================%
%设置ADC采集数据有效位宽
ADC_DATA_WIDTH = 16;
%ADC采集电压范围±VOL_RANGE (V)
VOL_RANGE = 5;
%采样率(Hz)
SAMPLE_RATE = 200000;
%ADC采集电压分辨率 LSB
LSB = VOL_RANGE*2/(2^ADC_DATA_WIDTH);
%====================================================================%
% ...（前面的代码保持不变）

% 读取文件数据，转换成波形显示
fileID = fopen('D:\DeskTop\udp.bin');
src_data = fread(fileID);
fclose(fileID);
src_data_hex = src_data;

DATA_NUM = length(src_data_hex);
if(DATA_NUM > 1024*64)
   DATA_NUM = 1024*64;
end

% 假设两个通道交替存储，每个通道的采样数为DATA_NUM/2
voltage_code = 1:DATA_NUM/2;
voltage_code = voltage_code';

for i=1:DATA_NUM/2
   if src_data_hex(2*i-1)+src_data_hex(i*2)*256 > 2^(ADC_DATA_WIDTH-1)
     voltage_code(i) = src_data_hex(2*i-1)+src_data_hex(i*2)*256-2^ADC_DATA_WIDTH;
   else
     voltage_code(i) = src_data_hex(2*i-1)+src_data_hex(i*2)*256;
   end
end

% 二进制ADC数据转换为电压值
voltage = voltage_code*LSB - LSB/2;

% ADC采样时间
data_8_1 = voltage(1:2:end,1);
data_num = length(data_8_1);   % 计算8组中一组的长度2048 

sample_t = 1:data_num;
sample_t = sample_t';
for i=1:data_num
   sample_t(i) = 1000000/SAMPLE_RATE * i;
end

% 绘制第一个通道的波形
subplot(2,2,1)
plot(sample_t,voltage(1:2:end,1));
axis([0 1000000/SAMPLE_RATE*data_num -VOL_RANGE VOL_RANGE]) 
title('ADC采集电压显示','Fontsize',12,'position',[12000,6])
xlabel('采样时间/(us)','Fontsize',10,'position',[9700,-7])
ylabel('电压/(V)','Fontsize',10)
legend('第一通道')
grid on

% 绘制第二个通道的波形
subplot(2,2,2)
plot(sample_t,voltage(2:2:end,1));
axis([0 1000000/SAMPLE_RATE*data_num -VOL_RANGE VOL_RANGE]) 
xlabel('采样时间/(us)','Fontsize',10,'position',[9700,-7])
ylabel('电压/(V)','Fontsize',10)
legend('第二通道')
grid on

% ...（FFT部分的代码也需要相应地只处理两个通道的数据）

% 双figure图形
figure
L = data_num/2;
NFFT = 2^nextpow2(L);

% 第一通道FFT
Y = fft(voltage(1:2:end,1),NFFT)/L;
Fs = SAMPLE_RATE;
f = Fs/2*linspace(0,1,NFFT/2+1);
len = max(f);
data = 2*abs(Y(1:NFFT/2+1));
subplot(2,2,1)
plot(f,data);
xlabel('Frequency (Hz)')
ylabel('|Y(f)|')
axis([0 len 0 5])
legend('第一通道')

% 第二通道FFT
Y = fft(voltage(2:2:end,1),NFFT)/L;
Fs = SAMPLE_RATE;
f = Fs/2*linspace(0,1,NFFT/2+1);
len = max(f);
data = 2*abs(Y(1:NFFT/2+1));
subplot(2,2,2)
plot(f,data);
xlabel('Frequency (Hz)')
ylabel('|Y(f)|')
axis([0 len 0 5])
legend('第二通道')

% ...（其他通道的FFT代码可以移除或注释掉）



